// Copyright lowRISC contributors.
// Licensed under the Apache License, Version 2.0, see LICENSE for details.
// SPDX-License-Identifier: Apache-2.0
//
// Register Top module auto-generated by `reggen`

`include "prim_assert.sv"

module aes_reg_top (
  input clk_i,
  input rst_ni,

  // Below Regster interface can be changed
  input  tlul_pkg::tl_h2d_t tl_i,
  output tlul_pkg::tl_d2h_t tl_o,
  // To HW
  output aes_reg_pkg::aes_reg2hw_t reg2hw, // Write
  input  aes_reg_pkg::aes_hw2reg_t hw2reg, // Read

  // Config
  input devmode_i // If 1, explicit error return for unmapped register access
);

  import aes_reg_pkg::* ;

  localparam int AW = 7;
  localparam int DW = 32;
  localparam int DBW = DW/8;                    // Byte Width

  // register signals
  logic           reg_we;
  logic           reg_re;
  logic [AW-1:0]  reg_addr;
  logic [DW-1:0]  reg_wdata;
  logic [DBW-1:0] reg_be;
  logic [DW-1:0]  reg_rdata;
  logic           reg_error;

  logic          addrmiss, wr_err;

  logic [DW-1:0] reg_rdata_next;

  tlul_pkg::tl_h2d_t tl_reg_h2d;
  tlul_pkg::tl_d2h_t tl_reg_d2h;

  assign tl_reg_h2d = tl_i;
  assign tl_o       = tl_reg_d2h;

  tlul_adapter_reg #(
    .RegAw(AW),
    .RegDw(DW)
  ) u_reg_if (
    .clk_i,
    .rst_ni,

    .tl_i (tl_reg_h2d),
    .tl_o (tl_reg_d2h),

    .we_o    (reg_we),
    .re_o    (reg_re),
    .addr_o  (reg_addr),
    .wdata_o (reg_wdata),
    .be_o    (reg_be),
    .rdata_i (reg_rdata),
    .error_i (reg_error)
  );

  assign reg_rdata = reg_rdata_next ;
  assign reg_error = (devmode_i & addrmiss) | wr_err ;

  // Define SW related signals
  // Format: <reg>_<field>_{wd|we|qs}
  //        or <reg>_{wd|we|qs} if field == 1 or 0
  logic [31:0] key0_wd;
  logic key0_we;
  logic [31:0] key1_wd;
  logic key1_we;
  logic [31:0] key2_wd;
  logic key2_we;
  logic [31:0] key3_wd;
  logic key3_we;
  logic [31:0] key4_wd;
  logic key4_we;
  logic [31:0] key5_wd;
  logic key5_we;
  logic [31:0] key6_wd;
  logic key6_we;
  logic [31:0] key7_wd;
  logic key7_we;
  logic [31:0] iv0_wd;
  logic iv0_we;
  logic [31:0] iv1_wd;
  logic iv1_we;
  logic [31:0] iv2_wd;
  logic iv2_we;
  logic [31:0] iv3_wd;
  logic iv3_we;
  logic [31:0] data_in0_wd;
  logic data_in0_we;
  logic [31:0] data_in1_wd;
  logic data_in1_we;
  logic [31:0] data_in2_wd;
  logic data_in2_we;
  logic [31:0] data_in3_wd;
  logic data_in3_we;
  logic [31:0] data_out0_qs;
  logic data_out0_re;
  logic [31:0] data_out1_qs;
  logic data_out1_re;
  logic [31:0] data_out2_qs;
  logic data_out2_re;
  logic [31:0] data_out3_qs;
  logic data_out3_re;
  logic ctrl_operation_qs;
  logic ctrl_operation_wd;
  logic ctrl_operation_we;
  logic ctrl_operation_re;
  logic [2:0] ctrl_mode_qs;
  logic [2:0] ctrl_mode_wd;
  logic ctrl_mode_we;
  logic ctrl_mode_re;
  logic [2:0] ctrl_key_len_qs;
  logic [2:0] ctrl_key_len_wd;
  logic ctrl_key_len_we;
  logic ctrl_key_len_re;
  logic ctrl_manual_operation_qs;
  logic ctrl_manual_operation_wd;
  logic ctrl_manual_operation_we;
  logic ctrl_manual_operation_re;
  logic trigger_start_wd;
  logic trigger_start_we;
  logic trigger_key_clear_wd;
  logic trigger_key_clear_we;
  logic trigger_iv_clear_wd;
  logic trigger_iv_clear_we;
  logic trigger_data_in_clear_wd;
  logic trigger_data_in_clear_we;
  logic trigger_data_out_clear_wd;
  logic trigger_data_out_clear_we;
  logic status_idle_qs;
  logic status_stall_qs;
  logic status_output_valid_qs;
  logic status_input_ready_qs;

  // Register instances

  // Subregister 0 of Multireg key
  // R[key0]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key0 (
    .re     (1'b0),
    .we     (key0_we),
    .wd     (key0_wd),
    .d      (hw2reg.key[0].d),
    .qre    (),
    .qe     (reg2hw.key[0].qe),
    .q      (reg2hw.key[0].q ),
    .qs     ()
  );

  // Subregister 1 of Multireg key
  // R[key1]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key1 (
    .re     (1'b0),
    .we     (key1_we),
    .wd     (key1_wd),
    .d      (hw2reg.key[1].d),
    .qre    (),
    .qe     (reg2hw.key[1].qe),
    .q      (reg2hw.key[1].q ),
    .qs     ()
  );

  // Subregister 2 of Multireg key
  // R[key2]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key2 (
    .re     (1'b0),
    .we     (key2_we),
    .wd     (key2_wd),
    .d      (hw2reg.key[2].d),
    .qre    (),
    .qe     (reg2hw.key[2].qe),
    .q      (reg2hw.key[2].q ),
    .qs     ()
  );

  // Subregister 3 of Multireg key
  // R[key3]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key3 (
    .re     (1'b0),
    .we     (key3_we),
    .wd     (key3_wd),
    .d      (hw2reg.key[3].d),
    .qre    (),
    .qe     (reg2hw.key[3].qe),
    .q      (reg2hw.key[3].q ),
    .qs     ()
  );

  // Subregister 4 of Multireg key
  // R[key4]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key4 (
    .re     (1'b0),
    .we     (key4_we),
    .wd     (key4_wd),
    .d      (hw2reg.key[4].d),
    .qre    (),
    .qe     (reg2hw.key[4].qe),
    .q      (reg2hw.key[4].q ),
    .qs     ()
  );

  // Subregister 5 of Multireg key
  // R[key5]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key5 (
    .re     (1'b0),
    .we     (key5_we),
    .wd     (key5_wd),
    .d      (hw2reg.key[5].d),
    .qre    (),
    .qe     (reg2hw.key[5].qe),
    .q      (reg2hw.key[5].q ),
    .qs     ()
  );

  // Subregister 6 of Multireg key
  // R[key6]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key6 (
    .re     (1'b0),
    .we     (key6_we),
    .wd     (key6_wd),
    .d      (hw2reg.key[6].d),
    .qre    (),
    .qe     (reg2hw.key[6].qe),
    .q      (reg2hw.key[6].q ),
    .qs     ()
  );

  // Subregister 7 of Multireg key
  // R[key7]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_key7 (
    .re     (1'b0),
    .we     (key7_we),
    .wd     (key7_wd),
    .d      (hw2reg.key[7].d),
    .qre    (),
    .qe     (reg2hw.key[7].qe),
    .q      (reg2hw.key[7].q ),
    .qs     ()
  );



  // Subregister 0 of Multireg iv
  // R[iv0]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_iv0 (
    .re     (1'b0),
    .we     (iv0_we),
    .wd     (iv0_wd),
    .d      (hw2reg.iv[0].d),
    .qre    (),
    .qe     (reg2hw.iv[0].qe),
    .q      (reg2hw.iv[0].q ),
    .qs     ()
  );

  // Subregister 1 of Multireg iv
  // R[iv1]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_iv1 (
    .re     (1'b0),
    .we     (iv1_we),
    .wd     (iv1_wd),
    .d      (hw2reg.iv[1].d),
    .qre    (),
    .qe     (reg2hw.iv[1].qe),
    .q      (reg2hw.iv[1].q ),
    .qs     ()
  );

  // Subregister 2 of Multireg iv
  // R[iv2]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_iv2 (
    .re     (1'b0),
    .we     (iv2_we),
    .wd     (iv2_wd),
    .d      (hw2reg.iv[2].d),
    .qre    (),
    .qe     (reg2hw.iv[2].qe),
    .q      (reg2hw.iv[2].q ),
    .qs     ()
  );

  // Subregister 3 of Multireg iv
  // R[iv3]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_iv3 (
    .re     (1'b0),
    .we     (iv3_we),
    .wd     (iv3_wd),
    .d      (hw2reg.iv[3].d),
    .qre    (),
    .qe     (reg2hw.iv[3].qe),
    .q      (reg2hw.iv[3].q ),
    .qs     ()
  );



  // Subregister 0 of Multireg data_in
  // R[data_in0]: V(False)

  prim_subreg #(
    .DW      (32),
    .SWACCESS("WO"),
    .RESVAL  (32'h0)
  ) u_data_in0 (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (data_in0_we),
    .wd     (data_in0_wd),

    // from internal hardware
    .de     (hw2reg.data_in[0].de),
    .d      (hw2reg.data_in[0].d ),

    // to internal hardware
    .qe     (reg2hw.data_in[0].qe),
    .q      (reg2hw.data_in[0].q ),

    .qs     ()
  );

  // Subregister 1 of Multireg data_in
  // R[data_in1]: V(False)

  prim_subreg #(
    .DW      (32),
    .SWACCESS("WO"),
    .RESVAL  (32'h0)
  ) u_data_in1 (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (data_in1_we),
    .wd     (data_in1_wd),

    // from internal hardware
    .de     (hw2reg.data_in[1].de),
    .d      (hw2reg.data_in[1].d ),

    // to internal hardware
    .qe     (reg2hw.data_in[1].qe),
    .q      (reg2hw.data_in[1].q ),

    .qs     ()
  );

  // Subregister 2 of Multireg data_in
  // R[data_in2]: V(False)

  prim_subreg #(
    .DW      (32),
    .SWACCESS("WO"),
    .RESVAL  (32'h0)
  ) u_data_in2 (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (data_in2_we),
    .wd     (data_in2_wd),

    // from internal hardware
    .de     (hw2reg.data_in[2].de),
    .d      (hw2reg.data_in[2].d ),

    // to internal hardware
    .qe     (reg2hw.data_in[2].qe),
    .q      (reg2hw.data_in[2].q ),

    .qs     ()
  );

  // Subregister 3 of Multireg data_in
  // R[data_in3]: V(False)

  prim_subreg #(
    .DW      (32),
    .SWACCESS("WO"),
    .RESVAL  (32'h0)
  ) u_data_in3 (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (data_in3_we),
    .wd     (data_in3_wd),

    // from internal hardware
    .de     (hw2reg.data_in[3].de),
    .d      (hw2reg.data_in[3].d ),

    // to internal hardware
    .qe     (reg2hw.data_in[3].qe),
    .q      (reg2hw.data_in[3].q ),

    .qs     ()
  );



  // Subregister 0 of Multireg data_out
  // R[data_out0]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_data_out0 (
    .re     (data_out0_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.data_out[0].d),
    .qre    (reg2hw.data_out[0].re),
    .qe     (),
    .q      (reg2hw.data_out[0].q ),
    .qs     (data_out0_qs)
  );

  // Subregister 1 of Multireg data_out
  // R[data_out1]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_data_out1 (
    .re     (data_out1_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.data_out[1].d),
    .qre    (reg2hw.data_out[1].re),
    .qe     (),
    .q      (reg2hw.data_out[1].q ),
    .qs     (data_out1_qs)
  );

  // Subregister 2 of Multireg data_out
  // R[data_out2]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_data_out2 (
    .re     (data_out2_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.data_out[2].d),
    .qre    (reg2hw.data_out[2].re),
    .qe     (),
    .q      (reg2hw.data_out[2].q ),
    .qs     (data_out2_qs)
  );

  // Subregister 3 of Multireg data_out
  // R[data_out3]: V(True)

  prim_subreg_ext #(
    .DW    (32)
  ) u_data_out3 (
    .re     (data_out3_re),
    .we     (1'b0),
    .wd     ('0),
    .d      (hw2reg.data_out[3].d),
    .qre    (reg2hw.data_out[3].re),
    .qe     (),
    .q      (reg2hw.data_out[3].q ),
    .qs     (data_out3_qs)
  );


  // R[ctrl]: V(True)

  //   F[operation]: 0:0
  prim_subreg_ext #(
    .DW    (1)
  ) u_ctrl_operation (
    .re     (ctrl_operation_re),
    .we     (ctrl_operation_we),
    .wd     (ctrl_operation_wd),
    .d      (hw2reg.ctrl.operation.d),
    .qre    (),
    .qe     (reg2hw.ctrl.operation.qe),
    .q      (reg2hw.ctrl.operation.q ),
    .qs     (ctrl_operation_qs)
  );


  //   F[mode]: 3:1
  prim_subreg_ext #(
    .DW    (3)
  ) u_ctrl_mode (
    .re     (ctrl_mode_re),
    .we     (ctrl_mode_we),
    .wd     (ctrl_mode_wd),
    .d      (hw2reg.ctrl.mode.d),
    .qre    (),
    .qe     (reg2hw.ctrl.mode.qe),
    .q      (reg2hw.ctrl.mode.q ),
    .qs     (ctrl_mode_qs)
  );


  //   F[key_len]: 6:4
  prim_subreg_ext #(
    .DW    (3)
  ) u_ctrl_key_len (
    .re     (ctrl_key_len_re),
    .we     (ctrl_key_len_we),
    .wd     (ctrl_key_len_wd),
    .d      (hw2reg.ctrl.key_len.d),
    .qre    (),
    .qe     (reg2hw.ctrl.key_len.qe),
    .q      (reg2hw.ctrl.key_len.q ),
    .qs     (ctrl_key_len_qs)
  );


  //   F[manual_operation]: 7:7
  prim_subreg_ext #(
    .DW    (1)
  ) u_ctrl_manual_operation (
    .re     (ctrl_manual_operation_re),
    .we     (ctrl_manual_operation_we),
    .wd     (ctrl_manual_operation_wd),
    .d      (hw2reg.ctrl.manual_operation.d),
    .qre    (),
    .qe     (reg2hw.ctrl.manual_operation.qe),
    .q      (reg2hw.ctrl.manual_operation.q ),
    .qs     (ctrl_manual_operation_qs)
  );


  // R[trigger]: V(False)

  //   F[start]: 0:0
  prim_subreg #(
    .DW      (1),
    .SWACCESS("WO"),
    .RESVAL  (1'h0)
  ) u_trigger_start (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (trigger_start_we),
    .wd     (trigger_start_wd),

    // from internal hardware
    .de     (hw2reg.trigger.start.de),
    .d      (hw2reg.trigger.start.d ),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.trigger.start.q ),

    .qs     ()
  );


  //   F[key_clear]: 1:1
  prim_subreg #(
    .DW      (1),
    .SWACCESS("WO"),
    .RESVAL  (1'h0)
  ) u_trigger_key_clear (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (trigger_key_clear_we),
    .wd     (trigger_key_clear_wd),

    // from internal hardware
    .de     (hw2reg.trigger.key_clear.de),
    .d      (hw2reg.trigger.key_clear.d ),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.trigger.key_clear.q ),

    .qs     ()
  );


  //   F[iv_clear]: 2:2
  prim_subreg #(
    .DW      (1),
    .SWACCESS("WO"),
    .RESVAL  (1'h0)
  ) u_trigger_iv_clear (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (trigger_iv_clear_we),
    .wd     (trigger_iv_clear_wd),

    // from internal hardware
    .de     (hw2reg.trigger.iv_clear.de),
    .d      (hw2reg.trigger.iv_clear.d ),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.trigger.iv_clear.q ),

    .qs     ()
  );


  //   F[data_in_clear]: 3:3
  prim_subreg #(
    .DW      (1),
    .SWACCESS("WO"),
    .RESVAL  (1'h0)
  ) u_trigger_data_in_clear (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (trigger_data_in_clear_we),
    .wd     (trigger_data_in_clear_wd),

    // from internal hardware
    .de     (hw2reg.trigger.data_in_clear.de),
    .d      (hw2reg.trigger.data_in_clear.d ),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.trigger.data_in_clear.q ),

    .qs     ()
  );


  //   F[data_out_clear]: 4:4
  prim_subreg #(
    .DW      (1),
    .SWACCESS("WO"),
    .RESVAL  (1'h0)
  ) u_trigger_data_out_clear (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    // from register interface
    .we     (trigger_data_out_clear_we),
    .wd     (trigger_data_out_clear_wd),

    // from internal hardware
    .de     (hw2reg.trigger.data_out_clear.de),
    .d      (hw2reg.trigger.data_out_clear.d ),

    // to internal hardware
    .qe     (),
    .q      (reg2hw.trigger.data_out_clear.q ),

    .qs     ()
  );


  // R[status]: V(False)

  //   F[idle]: 0:0
  prim_subreg #(
    .DW      (1),
    .SWACCESS("RO"),
    .RESVAL  (1'h1)
  ) u_status_idle (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    .we     (1'b0),
    .wd     ('0  ),

    // from internal hardware
    .de     (hw2reg.status.idle.de),
    .d      (hw2reg.status.idle.d ),

    // to internal hardware
    .qe     (),
    .q      (),

    // to register interface (read)
    .qs     (status_idle_qs)
  );


  //   F[stall]: 1:1
  prim_subreg #(
    .DW      (1),
    .SWACCESS("RO"),
    .RESVAL  (1'h0)
  ) u_status_stall (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    .we     (1'b0),
    .wd     ('0  ),

    // from internal hardware
    .de     (hw2reg.status.stall.de),
    .d      (hw2reg.status.stall.d ),

    // to internal hardware
    .qe     (),
    .q      (),

    // to register interface (read)
    .qs     (status_stall_qs)
  );


  //   F[output_valid]: 2:2
  prim_subreg #(
    .DW      (1),
    .SWACCESS("RO"),
    .RESVAL  (1'h0)
  ) u_status_output_valid (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    .we     (1'b0),
    .wd     ('0  ),

    // from internal hardware
    .de     (hw2reg.status.output_valid.de),
    .d      (hw2reg.status.output_valid.d ),

    // to internal hardware
    .qe     (),
    .q      (),

    // to register interface (read)
    .qs     (status_output_valid_qs)
  );


  //   F[input_ready]: 3:3
  prim_subreg #(
    .DW      (1),
    .SWACCESS("RO"),
    .RESVAL  (1'h1)
  ) u_status_input_ready (
    .clk_i   (clk_i    ),
    .rst_ni  (rst_ni  ),

    .we     (1'b0),
    .wd     ('0  ),

    // from internal hardware
    .de     (hw2reg.status.input_ready.de),
    .d      (hw2reg.status.input_ready.d ),

    // to internal hardware
    .qe     (),
    .q      (),

    // to register interface (read)
    .qs     (status_input_ready_qs)
  );




  logic [22:0] addr_hit;
  always_comb begin
    addr_hit = '0;
    addr_hit[ 0] = (reg_addr == AES_KEY0_OFFSET);
    addr_hit[ 1] = (reg_addr == AES_KEY1_OFFSET);
    addr_hit[ 2] = (reg_addr == AES_KEY2_OFFSET);
    addr_hit[ 3] = (reg_addr == AES_KEY3_OFFSET);
    addr_hit[ 4] = (reg_addr == AES_KEY4_OFFSET);
    addr_hit[ 5] = (reg_addr == AES_KEY5_OFFSET);
    addr_hit[ 6] = (reg_addr == AES_KEY6_OFFSET);
    addr_hit[ 7] = (reg_addr == AES_KEY7_OFFSET);
    addr_hit[ 8] = (reg_addr == AES_IV0_OFFSET);
    addr_hit[ 9] = (reg_addr == AES_IV1_OFFSET);
    addr_hit[10] = (reg_addr == AES_IV2_OFFSET);
    addr_hit[11] = (reg_addr == AES_IV3_OFFSET);
    addr_hit[12] = (reg_addr == AES_DATA_IN0_OFFSET);
    addr_hit[13] = (reg_addr == AES_DATA_IN1_OFFSET);
    addr_hit[14] = (reg_addr == AES_DATA_IN2_OFFSET);
    addr_hit[15] = (reg_addr == AES_DATA_IN3_OFFSET);
    addr_hit[16] = (reg_addr == AES_DATA_OUT0_OFFSET);
    addr_hit[17] = (reg_addr == AES_DATA_OUT1_OFFSET);
    addr_hit[18] = (reg_addr == AES_DATA_OUT2_OFFSET);
    addr_hit[19] = (reg_addr == AES_DATA_OUT3_OFFSET);
    addr_hit[20] = (reg_addr == AES_CTRL_OFFSET);
    addr_hit[21] = (reg_addr == AES_TRIGGER_OFFSET);
    addr_hit[22] = (reg_addr == AES_STATUS_OFFSET);
  end

  assign addrmiss = (reg_re || reg_we) ? ~|addr_hit : 1'b0 ;

  // Check sub-word write is permitted
  always_comb begin
    wr_err = 1'b0;
    if (addr_hit[ 0] && reg_we && (AES_PERMIT[ 0] != (AES_PERMIT[ 0] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 1] && reg_we && (AES_PERMIT[ 1] != (AES_PERMIT[ 1] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 2] && reg_we && (AES_PERMIT[ 2] != (AES_PERMIT[ 2] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 3] && reg_we && (AES_PERMIT[ 3] != (AES_PERMIT[ 3] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 4] && reg_we && (AES_PERMIT[ 4] != (AES_PERMIT[ 4] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 5] && reg_we && (AES_PERMIT[ 5] != (AES_PERMIT[ 5] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 6] && reg_we && (AES_PERMIT[ 6] != (AES_PERMIT[ 6] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 7] && reg_we && (AES_PERMIT[ 7] != (AES_PERMIT[ 7] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 8] && reg_we && (AES_PERMIT[ 8] != (AES_PERMIT[ 8] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[ 9] && reg_we && (AES_PERMIT[ 9] != (AES_PERMIT[ 9] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[10] && reg_we && (AES_PERMIT[10] != (AES_PERMIT[10] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[11] && reg_we && (AES_PERMIT[11] != (AES_PERMIT[11] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[12] && reg_we && (AES_PERMIT[12] != (AES_PERMIT[12] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[13] && reg_we && (AES_PERMIT[13] != (AES_PERMIT[13] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[14] && reg_we && (AES_PERMIT[14] != (AES_PERMIT[14] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[15] && reg_we && (AES_PERMIT[15] != (AES_PERMIT[15] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[16] && reg_we && (AES_PERMIT[16] != (AES_PERMIT[16] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[17] && reg_we && (AES_PERMIT[17] != (AES_PERMIT[17] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[18] && reg_we && (AES_PERMIT[18] != (AES_PERMIT[18] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[19] && reg_we && (AES_PERMIT[19] != (AES_PERMIT[19] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[20] && reg_we && (AES_PERMIT[20] != (AES_PERMIT[20] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[21] && reg_we && (AES_PERMIT[21] != (AES_PERMIT[21] & reg_be))) wr_err = 1'b1 ;
    if (addr_hit[22] && reg_we && (AES_PERMIT[22] != (AES_PERMIT[22] & reg_be))) wr_err = 1'b1 ;
  end

  assign key0_we = addr_hit[0] & reg_we & ~wr_err;
  assign key0_wd = reg_wdata[31:0];

  assign key1_we = addr_hit[1] & reg_we & ~wr_err;
  assign key1_wd = reg_wdata[31:0];

  assign key2_we = addr_hit[2] & reg_we & ~wr_err;
  assign key2_wd = reg_wdata[31:0];

  assign key3_we = addr_hit[3] & reg_we & ~wr_err;
  assign key3_wd = reg_wdata[31:0];

  assign key4_we = addr_hit[4] & reg_we & ~wr_err;
  assign key4_wd = reg_wdata[31:0];

  assign key5_we = addr_hit[5] & reg_we & ~wr_err;
  assign key5_wd = reg_wdata[31:0];

  assign key6_we = addr_hit[6] & reg_we & ~wr_err;
  assign key6_wd = reg_wdata[31:0];

  assign key7_we = addr_hit[7] & reg_we & ~wr_err;
  assign key7_wd = reg_wdata[31:0];

  assign iv0_we = addr_hit[8] & reg_we & ~wr_err;
  assign iv0_wd = reg_wdata[31:0];

  assign iv1_we = addr_hit[9] & reg_we & ~wr_err;
  assign iv1_wd = reg_wdata[31:0];

  assign iv2_we = addr_hit[10] & reg_we & ~wr_err;
  assign iv2_wd = reg_wdata[31:0];

  assign iv3_we = addr_hit[11] & reg_we & ~wr_err;
  assign iv3_wd = reg_wdata[31:0];

  assign data_in0_we = addr_hit[12] & reg_we & ~wr_err;
  assign data_in0_wd = reg_wdata[31:0];

  assign data_in1_we = addr_hit[13] & reg_we & ~wr_err;
  assign data_in1_wd = reg_wdata[31:0];

  assign data_in2_we = addr_hit[14] & reg_we & ~wr_err;
  assign data_in2_wd = reg_wdata[31:0];

  assign data_in3_we = addr_hit[15] & reg_we & ~wr_err;
  assign data_in3_wd = reg_wdata[31:0];

  assign data_out0_re = addr_hit[16] && reg_re;

  assign data_out1_re = addr_hit[17] && reg_re;

  assign data_out2_re = addr_hit[18] && reg_re;

  assign data_out3_re = addr_hit[19] && reg_re;

  assign ctrl_operation_we = addr_hit[20] & reg_we & ~wr_err;
  assign ctrl_operation_wd = reg_wdata[0];
  assign ctrl_operation_re = addr_hit[20] && reg_re;

  assign ctrl_mode_we = addr_hit[20] & reg_we & ~wr_err;
  assign ctrl_mode_wd = reg_wdata[3:1];
  assign ctrl_mode_re = addr_hit[20] && reg_re;

  assign ctrl_key_len_we = addr_hit[20] & reg_we & ~wr_err;
  assign ctrl_key_len_wd = reg_wdata[6:4];
  assign ctrl_key_len_re = addr_hit[20] && reg_re;

  assign ctrl_manual_operation_we = addr_hit[20] & reg_we & ~wr_err;
  assign ctrl_manual_operation_wd = reg_wdata[7];
  assign ctrl_manual_operation_re = addr_hit[20] && reg_re;

  assign trigger_start_we = addr_hit[21] & reg_we & ~wr_err;
  assign trigger_start_wd = reg_wdata[0];

  assign trigger_key_clear_we = addr_hit[21] & reg_we & ~wr_err;
  assign trigger_key_clear_wd = reg_wdata[1];

  assign trigger_iv_clear_we = addr_hit[21] & reg_we & ~wr_err;
  assign trigger_iv_clear_wd = reg_wdata[2];

  assign trigger_data_in_clear_we = addr_hit[21] & reg_we & ~wr_err;
  assign trigger_data_in_clear_wd = reg_wdata[3];

  assign trigger_data_out_clear_we = addr_hit[21] & reg_we & ~wr_err;
  assign trigger_data_out_clear_wd = reg_wdata[4];





  // Read data return
  always_comb begin
    reg_rdata_next = '0;
    unique case (1'b1)
      addr_hit[0]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[1]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[2]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[3]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[4]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[5]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[6]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[7]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[8]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[9]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[10]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[11]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[12]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[13]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[14]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[15]: begin
        reg_rdata_next[31:0] = '0;
      end

      addr_hit[16]: begin
        reg_rdata_next[31:0] = data_out0_qs;
      end

      addr_hit[17]: begin
        reg_rdata_next[31:0] = data_out1_qs;
      end

      addr_hit[18]: begin
        reg_rdata_next[31:0] = data_out2_qs;
      end

      addr_hit[19]: begin
        reg_rdata_next[31:0] = data_out3_qs;
      end

      addr_hit[20]: begin
        reg_rdata_next[0] = ctrl_operation_qs;
        reg_rdata_next[3:1] = ctrl_mode_qs;
        reg_rdata_next[6:4] = ctrl_key_len_qs;
        reg_rdata_next[7] = ctrl_manual_operation_qs;
      end

      addr_hit[21]: begin
        reg_rdata_next[0] = '0;
        reg_rdata_next[1] = '0;
        reg_rdata_next[2] = '0;
        reg_rdata_next[3] = '0;
        reg_rdata_next[4] = '0;
      end

      addr_hit[22]: begin
        reg_rdata_next[0] = status_idle_qs;
        reg_rdata_next[1] = status_stall_qs;
        reg_rdata_next[2] = status_output_valid_qs;
        reg_rdata_next[3] = status_input_ready_qs;
      end

      default: begin
        reg_rdata_next = '1;
      end
    endcase
  end

  // Assertions for Register Interface
  `ASSERT_PULSE(wePulse, reg_we)
  `ASSERT_PULSE(rePulse, reg_re)

  `ASSERT(reAfterRv, $rose(reg_re || reg_we) |=> tl_o.d_valid)

  `ASSERT(en2addrHit, (reg_we || reg_re) |-> $onehot0(addr_hit))

  // this is formulated as an assumption such that the FPV testbenches do disprove this
  // property by mistake
  `ASSUME(reqParity, tl_reg_h2d.a_valid |-> tl_reg_h2d.a_user.parity_en == 1'b0)

endmodule
